import logging
from werkzeug.exceptions import BadRequest
from odoo.addons.base_rest.http import wrapJsonException
from odoo.addons.component.core import Component
from . import schemas

_logger = logging.getLogger(__name__)


class CRMLeadService(Component):
    _inherit = "base.rest.service"
    _name = "crm.lead.services"
    _usage = "crm-lead"
    _collection = "emc.services"
    _description = """
        CRMLead requests
    """

    def create(self, **params):
        params = self._prepare_create(params)
        # tracking_disable=True in context is needed
        # to avoid to send a mail in CRMLead creation
        sr = self.env["crm.lead"].with_context(tracking_disable=True).create(params)
        return self._to_dict(sr)

    def _validator_create(self):
        return schemas.S_CRM_LEAD_CREATE

    def _validator_return_create(self):
        return schemas.S_CRM_LEAD_RETURN_CREATE

    @staticmethod
    def _to_dict(crm_lead):
        return {
            "id": crm_lead.id
        }

    def _get_country(self, code):
        country = self.env["res.country"].search([("code", "=", code)])
        if country:
            return country
        else:
            raise wrapJsonException(
                BadRequest("No country for isocode %s" % code)
            )

    def _get_state(self, code, country_id):
        state = self.env["res.country.state"].search([
            ("code", "=", code),
            ("country_id", "=", country_id)])
        if state:
            return state
        else:
            raise wrapJsonException(
                BadRequest(
                    "No state for isocode %s and country id %s" %
                    (code, str(country_id))
                )
            )

    def _prepare_create_address(self, address):
        prepared_address = address.copy()
        prepared_address["country_id"] = self._get_country(address["country"]).id
        prepared_address["state_id"] = self._get_state(
            address["state"],
            prepared_address["country"]).id
        prepared_address.pop("country")
        prepared_address.pop("state")
        return prepared_address

    def _prepare_create_isp_info(self, isp_info):
        Partner = self.env["res.partner"]

        isp_info["delivery_address"] = Partner.create(
            self._prepare_create_address(isp_info["delivery_address"])).id

        if "service_address" in isp_info.keys():
            isp_info["service_address"] = Partner.create(
                self._prepare_create_address(isp_info["service_address"])).id

        return isp_info

    def _prepare_create_line(self, line):
        product = self.env["product.product"].search(
            [('default_code', '=', line["product_code"])]
        )
        if not product:
            raise wrapJsonException(
                BadRequest(
                    'Product with code %s not found' % (
                        line['product_code'], )
                )
            )
        response_line = {
            "name": product.name,
            "product_id": product.id,
            "product_tmpl_id": product.product_tmpl_id.id,
            "category_id": product.categ_id.id
        }
        if line.get("broadband_isp_info"):
            response_line["broadband_isp_info"] = self.env["broadband.isp.info"].create(
                self._prepare_create_isp_info(line["broadband_isp_info"])).id
        elif not self._needs_mobile_isp_info(product):
            raise wrapJsonException(
                BadRequest(
                    'Broadband product %s needs a broadband_isp_info' % (
                        line['product_code'], )
                )
            )

        if line.get("mobile_isp_info"):
            response_line["mobile_isp_info"] = self.env["mobile.isp.info"].create(
                self._prepare_create_isp_info(line["mobile_isp_info"])).id
        elif self._needs_mobile_isp_info(product.product_tmpl_id):
            raise wrapJsonException(
                BadRequest(
                    'Mobile product %s needs a mobile_isp_info' % (
                        line['product_code'], )
                )
            )
        return response_line

    def _needs_mobile_isp_info(self, product_tmpl):
        mobile = self.env.ref('somconnexio.mobile_service')
        if mobile.id == product_tmpl.categ_id.id:
            return True

    def _prepare_create(self, params):
        if (
            'subscription_request_id' in params
            and not self.env['subscription.request'].search(
                [('id', '=', params['subscription_request_id'])])
        ):
            raise wrapJsonException(
                BadRequest(
                    'SubscriptionRequest with id %s not found' % (
                        params['subscription_request_id'], )
                )
            )
        elif (
            'partner_id' in params
            and not self.env['res.partner'].browse(params['partner_id'])
        ):
            raise wrapJsonException(
                BadRequest(
                    'Partner with id %s not found' % (
                        params['partner_id'], )
                )
            )

        crm_line_ids = [
            self.env["crm.lead.line"].create(self._prepare_create_line(line)).id
            for line in params["lead_line_ids"]
        ]
        return {
            # TODO: What do we want to put in the CRMLead name and CRMLeadName?
            "name": "TODO",
            "partner_id": params.get("partner_id"),
            "iban": params.get("iban"),
            "subscription_request_id": params.get("subscription_request_id"),
            "lead_line_ids": [(6, 0, crm_line_ids)]
        }
